@@ -168,7 +168,8 @@ extern void  amd64g_dirtyhelper_CPUID_baseline ( VexGuestAMD64State* st );
 extern void  amd64g_dirtyhelper_CPUID_sse3_and_cx16 ( VexGuestAMD64State* st );
 extern void  amd64g_dirtyhelper_CPUID_sse42_and_cx16 ( VexGuestAMD64State* st );
 extern void  amd64g_dirtyhelper_CPUID_avx_and_cx16 ( VexGuestAMD64State* st );
-extern void  amd64g_dirtyhelper_CPUID_avx2 ( VexGuestAMD64State* st );
+extern void  amd64g_dirtyhelper_CPUID_avx2 ( VexGuestAMD64State* st,
+                                             ULong hasF16C, ULong hasRDRAND );
 
 extern void  amd64g_dirtyhelper_FINIT ( VexGuestAMD64State* );
 
@@ -192,6 +193,11 @@ extern void  amd64g_dirtyhelper_OUT ( ULong portno, ULong data,
 extern void amd64g_dirtyhelper_SxDT ( void* address,
                                       ULong op /* 0 or 1 */ );
 
+// This returns a 32-bit value from the host's RDRAND in bits 31:0, and the
+// resulting C flag value in bit 32.
+extern ULong amd64g_dirtyhelper_RDRAND ( void );
+
+
 /* Helps with PCMP{I,E}STR{I,M}.
 
    CALLED FROM GENERATED CODE: DIRTY HELPER(s).  (But not really,
