@@ -1052,10 +1052,14 @@ Bool compute_PCMPxSTRx_wide ( /*OUT*/V128* resV,
       even if they would probably work.  Life is too short to have
       unvalidated cases in the code base. */
    switch (imm8) {
-      case 0x01: case 0x03: case 0x09: case 0x0B: case 0x0D:
-                 case 0x13: case 0x19: case 0x1B:
-                            case 0x39: case 0x3B:
-                 case 0x45:            case 0x4B:
+      //    1,9        3,B        5,D        7,F
+      case 0x01: case 0x03:
+      case 0x09: case 0x0B: case 0x0D:
+                 case 0x13:
+      case 0x19: case 0x1B:
+      case 0x39: case 0x3B:
+      case 0x41:            case 0x45:
+                 case 0x4B:
          break;
       default:
          return False;
