@@ -6,7 +6,7 @@
 /*
    This file is part of Callgrind, a Valgrind tool for call tracing.
 
-   Copyright (C) 2002-2015, Josef Weidendorfer (Josef.Weidendorfer@gmx.de)
+   Copyright (C) 2002-2017, Josef Weidendorfer (Josef.Weidendorfer@gmx.de)
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -100,8 +100,10 @@ void CLG_(zero_bbcc)(BBCC* bbcc)
     bbcc->cost[i] = 0;
   for(i=0;i <= bbcc->bb->cjmp_count;i++) {
     bbcc->jmp[i].ecounter = 0;
-    for(jcc=bbcc->jmp[i].jcc_list; jcc; jcc=jcc->next_from)
-	CLG_(init_cost)( CLG_(sets).full, jcc->cost );
+    for(jcc=bbcc->jmp[i].jcc_list; jcc; jcc=jcc->next_from) {
+      CLG_(init_cost)( CLG_(sets).full, jcc->cost );
+      jcc->call_counter = 0;
+    }
   }
   bbcc->ecounter_sum = 0;
   bbcc->ret_counter = 0;
