@@ -8,7 +8,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2008-2015 OpenWorks Ltd
+   Copyright (C) 2008-2017 OpenWorks Ltd
       info@open-works.co.uk
 
    This program is free software; you can redistribute it and/or
@@ -86,7 +86,7 @@ static ThreadId find_tid_with_stack_containing (Addr a)
    return VG_INVALID_THREADID;
 }
 
-void VG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
+void VG_(describe_addr) ( DiEpoch ep, Addr a, /*OUT*/AddrInfo* ai )
 {
    VgSectKind sect;
 
@@ -99,7 +99,7 @@ void VG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
                     VG_(free), sizeof(HChar) );
 
    (void) VG_(get_data_description)( ai->Addr.Variable.descr1,
-                                     ai->Addr.Variable.descr2, a );
+                                     ai->Addr.Variable.descr2, ep, a );
    /* If there's nothing in descr1/2, free them.  Why is it safe to
       VG_(indexXA) at zero here?  Because VG_(get_data_description)
       guarantees to zero terminate descr1/2 regardless of the outcome
@@ -127,7 +127,7 @@ void VG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
       there. -- */
    const HChar *name;
    if (VG_(get_datasym_and_offset)(
-             a, &name,
+             ep, a, &name,
              &ai->Addr.DataSym.offset )) {
       ai->Addr.DataSym.name = VG_(strdup)("mc.da.dsname", name);
       ai->tag = Addr_DataSym;
@@ -148,6 +148,7 @@ void VG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
             ai->tag            = Addr_Stack;
             VG_(initThreadInfo)(&ai->Addr.Stack.tinfo);
             ai->Addr.Stack.tinfo.tid = tid;
+            ai->Addr.Stack.epoch = ep;
             ai->Addr.Stack.IP = 0;
             ai->Addr.Stack.frameNo = -1;
             ai->Addr.Stack.stackPos = StackPos_stacked;
@@ -222,7 +223,7 @@ void VG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
       ThreadId   tid;
       StackPos stackPos = StackPos_stacked;
       // Default init to StackPos_stacked, to silence gcc warning.
-      // We assert this value is overriden if a stack descr is produced.
+      // We assert this value is overridden if a stack descr is produced.
 
       // First try to find a tid with stack containing a
       tid = find_tid_with_stack_containing (a);
@@ -248,6 +249,7 @@ void VG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
          ai->tag  = Addr_Stack;
          VG_(initThreadInfo)(&ai->Addr.Stack.tinfo);
          ai->Addr.Stack.tinfo.tid = tid;
+         ai->Addr.Stack.epoch = ep;
          ai->Addr.Stack.IP = 0;
          ai->Addr.Stack.frameNo = -1;
          vg_assert (stackPos != StackPos_stacked);
@@ -447,23 +449,27 @@ static void pp_addrinfo_WRK ( Addr a, const AddrInfo* ai, Bool mc,
             Bool haslinenum;
             PtrdiffT offset;
 
-            if (VG_(get_inst_offset_in_function)( ai->Addr.Stack.IP,
+            if (VG_(get_inst_offset_in_function)( ai->Addr.Stack.epoch,
+                                                  ai->Addr.Stack.IP,
                                                   &offset))
-               haslinenum = VG_(get_linenum) (ai->Addr.Stack.IP - offset,
+               haslinenum = VG_(get_linenum) (ai->Addr.Stack.epoch,
+                                              ai->Addr.Stack.IP - offset,
                                               &linenum);
             else
                haslinenum = False;
 
-            hasfile = VG_(get_filename)(ai->Addr.Stack.IP, &file);
+            hasfile = VG_(get_filename)(ai->Addr.Stack.epoch,
+                                        ai->Addr.Stack.IP, &file);
 
             HChar strlinenum[16] = "";   // large enough
             if (hasfile && haslinenum)
                VG_(sprintf)(strlinenum, "%u", linenum);
 
-            hasfn = VG_(get_fnname)(ai->Addr.Stack.IP, &fn);
+            hasfn = VG_(get_fnname)(ai->Addr.Stack.epoch,
+                                    ai->Addr.Stack.IP, &fn);
 
             if (hasfn || hasfile)
-               VG_(emit)( "%sin frame #%d, created by %s (%s:%s)%s\n",
+               VG_(emit)( "%sin frame #%d, created by %ps (%ps:%s)%s\n",
                           xpre,
                           ai->Addr.Stack.frameNo, 
                           hasfn ? fn : "???", 
@@ -573,7 +579,7 @@ static void pp_addrinfo_WRK ( Addr a, const AddrInfo* ai, Bool mc,
 
       case Addr_DataSym:
          VG_(emit)( "%sAddress 0x%lx is %llu bytes "
-                    "inside data symbol \"%pS\"%s\n",
+                    "inside data symbol \"%ps\"%s\n",
                     xpre, a,
                     (ULong)ai->Addr.DataSym.offset,
                     ai->Addr.DataSym.name,
@@ -595,7 +601,7 @@ static void pp_addrinfo_WRK ( Addr a, const AddrInfo* ai, Bool mc,
          break;
 
       case Addr_SectKind:
-         VG_(emit)( "%sAddress 0x%lx is in the %pS segment of %pS%s\n",
+         VG_(emit)( "%sAddress 0x%lx is in the %ps segment of %ps%s\n",
                     xpre, a,
                     VG_(pp_SectKind)(ai->Addr.SectKind.kind),
                     ai->Addr.SectKind.objname,
@@ -603,7 +609,7 @@ static void pp_addrinfo_WRK ( Addr a, const AddrInfo* ai, Bool mc,
          if (ai->Addr.SectKind.kind == Vg_SectText) {
             /* To better describe the address in a text segment,
                pp a dummy stacktrace made of this single address. */
-            VG_(pp_StackTrace)( &a, 1 );
+            VG_(pp_StackTrace)( VG_(current_DiEpoch)(), &a, 1 );
          }
          break;
 
@@ -627,7 +633,7 @@ static void pp_addrinfo_WRK ( Addr a, const AddrInfo* ai, Bool mc,
 
       case Addr_SegmentKind:
          VG_(emit)( "%sAddress 0x%lx is in "
-                    "a %s%s%s %s%s%pS segment%s\n",
+                    "a %s%s%s %s%s%ps segment%s\n",
                     xpre,
                     a,
                     ai->Addr.SegmentKind.hasR ? "r" : "-",
