@@ -8,7 +8,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2002-2015 Nicholas Nethercote
+   Copyright (C) 2002-2017 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -540,8 +540,7 @@ get_cache_info(VexArchInfo *vai)
 
 #elif defined(VGA_arm) || defined(VGA_ppc32)    || \
    defined(VGA_ppc64be) || defined(VGA_ppc64le) || \
-   defined(VGA_mips32) || defined(VGA_mips64) || defined(VGA_arm64) || \
-   defined(VGA_tilegx)
+   defined(VGA_mips32) || defined(VGA_mips64) || defined(VGA_arm64)
 static Bool
 get_cache_info(VexArchInfo *vai)
 {
