@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2013-2015 Ivo Raisr
+   Copyright (C) 2013-2017 Ivo Raisr
       ivosh@ivosh.net
 
    This program is free software; you can redistribute it and/or
@@ -682,7 +682,7 @@ static vki_prpriv_t *create_prpriv(SizeT *size)
 
 static vki_priv_impl_info_t *create_priv_info(SizeT *size)
 {
-   /* Size of the returned priv_impl_info_t is apriori unkown. */
+   /* Size of the returned priv_impl_info_t is apriori unknown. */
    vki_priv_impl_info_t first_cut[100];
    SysRes sres = VG_(do_syscall5)(SYS_privsys, VKI_PRIVSYS_GETIMPLINFO,
                                   0, 0, (UWord) first_cut,
@@ -866,10 +866,10 @@ void VG_(make_coredump)(ThreadId tid, const vki_siginfo_t *si,
    const HChar *coreext = "";
    Int core_fd;
 
-   if (VG_(clo_log_fname_expanded) != NULL) {
+   if (VG_(clo_log_fname_unexpanded) != NULL) {
       coreext = ".core";
       basename = VG_(expand_file_name)("--log-file",
-                                       VG_(clo_log_fname_expanded));
+                                       VG_(clo_log_fname_unexpanded));
    }
 
    vg_assert(coreext != NULL);
