@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2008-2013 OpenWorks LLP
+   Copyright (C) 2008-2017 OpenWorks LLP
       info@open-works.co.uk
 
    This program is free software; you can redistribute it and/or
@@ -46,7 +46,6 @@ void* ML_(dinfo_zalloc) ( const HChar* cc, SizeT szB ) {
    void* v;
    vg_assert(szB > 0);
    v = VG_(arena_malloc)( VG_AR_DINFO, cc, szB );
-   vg_assert(v);
    VG_(memset)(v, 0, szB);
    return v;
 }
@@ -63,20 +62,25 @@ HChar* ML_(dinfo_strdup) ( const HChar* cc, const HChar* str ) {
    return VG_(arena_strdup)( VG_AR_DINFO, cc, str );
 }
 
-void* ML_(dinfo_memdup) ( const HChar* cc, void* str, SizeT nStr ) {
+void* ML_(dinfo_memdup) ( const HChar* cc, const void* str, SizeT nStr ) {
    void* dst = VG_(arena_malloc)( VG_AR_DINFO, cc, nStr );
-   tl_assert(dst);
    VG_(memcpy)(dst, str, nStr);
    return dst;
 }
 
+void* ML_(dinfo_realloc) ( const HChar* cc, void* ptr, SizeT new_size ) {
+   void* dst = VG_(arena_realloc)( VG_AR_DINFO, cc, ptr, new_size );
+   vg_assert(dst);
+   return dst;
+}
+
 static inline Bool host_is_little_endian ( void ) {
    UInt x = 0x76543210;
    UChar* p = (UChar*)(&x);
    return toBool(*p == 0x10);
 }
 
-Short ML_(read_Short)( UChar* data ) {
+Short ML_(readUAS_Short)( const UChar* data ) {
    Short r = 0;
    if (host_is_little_endian()) {
       r = data[0]
@@ -88,7 +92,7 @@ Short ML_(read_Short)( UChar* data ) {
    return r;
 }
 
-Int ML_(read_Int) ( UChar* data ) {
+Int ML_(readUAS_Int) ( const UChar* data ) {
    Int r = 0;
    if (host_is_little_endian()) {
       r = data[0]
@@ -104,7 +108,7 @@ Int ML_(read_Int) ( UChar* data ) {
    return r;
 }
 
-Long ML_(read_Long) ( UChar* data ) {
+Long ML_(readUAS_Long) ( const UChar* data ) {
    Long r = 0;
    if (host_is_little_endian()) {
       r = data[0]
@@ -128,7 +132,7 @@ Long ML_(read_Long) ( UChar* data ) {
    return r;
 }
 
-UShort ML_(read_UShort) ( UChar* data ) {
+UShort ML_(readUAS_UShort) ( const UChar* data ) {
    UInt r = 0;
    if (host_is_little_endian()) {
       r = data[0]
@@ -140,7 +144,7 @@ UShort ML_(read_UShort) ( UChar* data ) {
    return r;
 }
 
-UChar *ML_(write_UShort) ( UChar* ptr, UShort val ) {
+UChar *ML_(writeUAS_UShort) ( UChar* ptr, UShort val ) {
    if (host_is_little_endian()) {
       ptr[0] = val & 0xff;
       ptr[1] = ( val >> 8 ) & 0xff;
@@ -151,7 +155,7 @@ UChar *ML_(write_UShort) ( UChar* ptr, UShort val ) {
    return ptr + sizeof(UShort);
 }
 
-UWord ML_(read_UWord) ( UChar* data ) {
+UWord ML_(readUAS_UWord) ( const UChar* data ) {
    if (sizeof(UWord) == sizeof(UInt)) {
       return ML_(read_UInt)(data);
    } else if  (sizeof(UWord) == sizeof(ULong)) {
@@ -161,7 +165,7 @@ UWord ML_(read_UWord) ( UChar* data ) {
    }
 }
 
-UInt ML_(read_UInt) ( UChar* data ) {
+UInt ML_(readUAS_UInt) ( const UChar* data ) {
    UInt r = 0;
    if (host_is_little_endian()) {
       r = data[0]
@@ -177,7 +181,7 @@ UInt ML_(read_UInt) ( UChar* data ) {
    return r;
 }
 
-UChar* ML_(write_UInt) ( UChar* ptr, UInt val ) {
+UChar* ML_(writeUAS_UInt) ( UChar* ptr, UInt val ) {
    if (host_is_little_endian()) {
       ptr[0] = val & 0xff;
       ptr[1] = ( val >> 8 ) & 0xff;
@@ -192,7 +196,7 @@ UChar* ML_(write_UInt) ( UChar* ptr, UInt val ) {
    return ptr + sizeof(UInt);
 }
 
-ULong ML_(read_ULong) ( UChar* data ) {
+ULong ML_(readUAS_ULong) ( const UChar* data ) {
    ULong r = 0;
    if (host_is_little_endian()) {
       r = data[0]
@@ -216,7 +220,7 @@ ULong ML_(read_ULong) ( UChar* data ) {
    return r;
 }
 
-UChar* ML_(write_ULong) ( UChar* ptr, ULong val ) {
+UChar* ML_(writeUAS_ULong) ( UChar* ptr, ULong val ) {
    if (host_is_little_endian()) {
       ptr[0] = val & 0xff;
       ptr[1] = ( val >> 8 ) & 0xff;
@@ -239,16 +243,8 @@ UChar* ML_(write_ULong) ( UChar* ptr, ULong val ) {
    return ptr + sizeof(ULong);
 }
 
-UChar ML_(read_UChar) ( UChar* data ) {
-   return data[0];
-}
 
-UChar* ML_(write_UChar) ( UChar* ptr, UChar val ) {
-   ptr[0] = val;
-   return ptr + sizeof(UChar);
-}
-
-Addr ML_(read_Addr) ( UChar* data ) {
+Addr ML_(readUAS_Addr) ( const UChar* data ) {
    if (sizeof(Addr) == sizeof(UInt)) {
       return ML_(read_UInt)(data);
    } else if  (sizeof(Addr) == sizeof(ULong)) {
@@ -258,7 +254,7 @@ Addr ML_(read_Addr) ( UChar* data ) {
    }
 }
 
-UChar* ML_(write_Addr) ( UChar* ptr, Addr val ) {
+UChar* ML_(writeUAS_Addr) ( UChar* ptr, Addr val ) {
    if (sizeof(Addr) == sizeof(UInt)) {
       return ML_(write_UInt)(ptr, val);
    } else if  (sizeof(Addr) == sizeof(ULong)) {
@@ -268,7 +264,6 @@ UChar* ML_(write_Addr) ( UChar* ptr, Addr val ) {
    }
 }
 
-
 /*--------------------------------------------------------------------*/
 /*--- end                                                   misc.c ---*/
 /*--------------------------------------------------------------------*/
