@@ -6,7 +6,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2014-2015 Philippe Waroquiers philippe.waroquiers@skynet.be
+   Copyright (C) 2014-2017 Philippe Waroquiers philippe.waroquiers@skynet.be
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -41,10 +41,11 @@
 struct _DedupPoolAlloc {
    SizeT  poolSzB; /* Minimum size of a pool. */
    SizeT  fixedSzb; /* If using VG_(allocFixedEltDedupPA), size of elements */
+   Bool   strPA;    /* True if this is a string dedup pool */
    SizeT  eltAlign;
-   void*   (*alloc_fn)(const HChar*, SizeT); /* pool allocator */
+   Alloc_Fn_t alloc_fn; /* pool allocator */
    const HChar*  cc; /* pool allocator's cost centre */
-   void    (*free_fn)(void*); /* pool allocator's deallocation function */
+   Free_Fn_t free_fn; /* pool allocator's deallocation function */
    /* XArray of void* (pointers to pools).  The pools themselves.
       Each element is a pointer to a block of size at least PoolSzB bytes.
       The last block might be smaller due to a call to shrink_block. */
@@ -75,16 +76,17 @@ typedef
    struct _ht_node {
       struct _ht_node *next; // Read/Write by hashtable (pub_tool_hashtable.h)
       UWord   key;           // Read by hashtable (pub_tool_hashtable.h)
-      SizeT   eltSzB;
+      SizeT   eltSzBorStrNr; // for a normal pool, elt size 
+                             // for a string pool, the unique str number
       const void *elt;
    }
    ht_node;
 
 DedupPoolAlloc* VG_(newDedupPA) ( SizeT  poolSzB,
                                   SizeT  eltAlign,
-                                  void*  (*alloc_fn)(const HChar*, SizeT),
+                                  Alloc_Fn_t alloc_fn,
                                   const  HChar* cc,
-                                  void   (*free_fn)(void*) )
+                                  Free_Fn_t free_fn )
 {
    DedupPoolAlloc* ddpa;
    vg_assert(poolSzB >= eltAlign);
@@ -97,6 +99,7 @@ DedupPoolAlloc* VG_(newDedupPA) ( SizeT  poolSzB,
    VG_(memset)(ddpa, 0, sizeof(*ddpa));
    ddpa->poolSzB  = poolSzB;
    ddpa->fixedSzb = 0;
+   ddpa->strPA = False;
    ddpa->eltAlign = eltAlign;
    ddpa->alloc_fn = alloc_fn;
    ddpa->cc       = cc;
@@ -189,14 +192,24 @@ static Word cmp_pool_elt (const void* node1, const void* node2 )
    /* As this function is called by hashtable, that has already checked
       for key equality, it is likely that it is the 'good' element.
       So, we handle the equal case first. */
-   if (hnode1->eltSzB == hnode2->eltSzB)
-      return VG_(memcmp) (hnode1->elt, hnode2->elt, hnode1->eltSzB);
-   else if (hnode1->eltSzB < hnode2->eltSzB)
+   if (hnode1->eltSzBorStrNr == hnode2->eltSzBorStrNr)
+      return VG_(memcmp) (hnode1->elt, hnode2->elt, hnode1->eltSzBorStrNr);
+   else if (hnode1->eltSzBorStrNr < hnode2->eltSzBorStrNr)
       return -1;
    else
       return 1;
 }
 
+/* String compare function for 'gen' hash table.
+   Similarly to cmp_pool_elt, no need to compare the key. */
+static Word cmp_pool_str (const void* node1, const void* node2 )
+{
+   const ht_node* hnode1 = node1;
+   const ht_node* hnode2 = node2;
+
+   return VG_(strcmp)(hnode1->elt, hnode2->elt);
+}
+
 /* Print some stats. */
 static void print_stats (DedupPoolAlloc *ddpa)
 {
@@ -209,7 +222,10 @@ static void print_stats (DedupPoolAlloc *ddpa)
                 VG_(sizeXA)(ddpa->pools),
                 ddpa->curpool ?
                 (long int) (ddpa->curpool_limit - ddpa->curpool_free + 1) : 0);
-   VG_(HT_print_stats) (ddpa->ht_elements, cmp_pool_elt);
+   if (ddpa->strPA)
+      VG_(HT_print_stats) (ddpa->ht_elements, cmp_pool_str);
+   else
+      VG_(HT_print_stats) (ddpa->ht_elements, cmp_pool_elt);
 }
 
 /* Dummy free, as the ht elements are allocated in a pool, and
@@ -247,46 +263,85 @@ static UInt sdbm_hash (const UChar* buf, UInt len )
   return h;
 }
 
-const void* VG_(allocEltDedupPA) (DedupPoolAlloc *ddpa, SizeT eltSzB,
-                                  const void *elt)
+static ht_node* allocEltDedupPA (DedupPoolAlloc *ddpa, SizeT eltSzB,
+                                 const void *elt)
 {
    ht_node ht_elt;
    void* elt_ins;
    ht_node *ht_ins;
    vg_assert(ddpa);
    vg_assert(ddpa->ht_elements);
-   vg_assert (eltSzB <= ddpa->poolSzB);
 
    ddpa->nr_alloc_calls++;
 
    ht_elt.key = sdbm_hash (elt, eltSzB);
 
-   ht_elt.eltSzB = eltSzB;
    ht_elt.elt = elt;
 
-   ht_ins = VG_(HT_gen_lookup) (ddpa->ht_elements, &ht_elt, cmp_pool_elt);
+   if (ddpa->strPA)
+      ht_ins = VG_(HT_gen_lookup) (ddpa->ht_elements, &ht_elt, cmp_pool_str);
+   else {
+      ht_elt.eltSzBorStrNr = eltSzB;
+      ht_ins = VG_(HT_gen_lookup) (ddpa->ht_elements, &ht_elt, cmp_pool_elt);
+   }
    if (ht_ins)
-      return ht_ins->elt;
+      return ht_ins;
 
    /* Not found -> we need to allocate a new element from the pool
       and insert it in the hash table of inserted elements. */
 
    // Add a new pool or grow pool if not enough space in the current pool
-   if (UNLIKELY(ddpa->curpool_free == NULL
-                || ddpa->curpool_free + eltSzB - 1 > ddpa->curpool_limit)) {
-      ddpa_add_new_pool_or_grow (ddpa);
+   if (eltSzB + ddpa->eltAlign > ddpa->poolSzB) {
+      // Element (+eltAlign for worst case) bigger than the pool size
+      // => allocate a specific pool just for this element
+      UChar *newpool = ddpa->alloc_fn (ddpa->cc, eltSzB + ddpa->eltAlign);
+      /* add to our collection of pools */
+      VG_(addToXA)( ddpa->pools, &newpool );
+      elt_ins = ddpa_align (ddpa, newpool);
+   } else {
+      if (UNLIKELY(ddpa->curpool_free == NULL
+                   || ddpa->curpool_free + eltSzB - 1 > ddpa->curpool_limit)) {
+         ddpa_add_new_pool_or_grow (ddpa);
+      }
+      elt_ins = ddpa->curpool_free;
+      ddpa->curpool_free = ddpa_align(ddpa, ddpa->curpool_free + eltSzB);
    }
 
-   elt_ins = ddpa->curpool_free;
-   VG_(memcpy)(elt_ins, elt, eltSzB);
-   ddpa->curpool_free = ddpa_align(ddpa, ddpa->curpool_free + eltSzB);
 
+   VG_(memcpy)(elt_ins, elt, eltSzB);
    ht_ins = VG_(allocEltPA) (ddpa->ht_node_pa);
    ht_ins->key = ht_elt.key;
-   ht_ins->eltSzB = eltSzB;
+   if (ddpa->strPA)
+      ht_ins->eltSzBorStrNr = VG_(HT_count_nodes)(ddpa->ht_elements) + 1;
+   else
+      ht_ins->eltSzBorStrNr = eltSzB;
    ht_ins->elt = elt_ins;
    VG_(HT_add_node)(ddpa->ht_elements, ht_ins);
-   return elt_ins;
+   return ht_ins;
+}
+
+const void* VG_(allocEltDedupPA) (DedupPoolAlloc *ddpa, SizeT eltSzB,
+                                  const void *elt)
+{
+   return allocEltDedupPA(ddpa, eltSzB, elt)->elt;
+}
+
+UInt VG_(allocStrDedupPA) (DedupPoolAlloc *ddpa,
+                           const HChar* str,
+                           Bool* newStr)
+{
+   if (!ddpa->strPA) {
+      // First insertion in this ddpa
+      vg_assert (ddpa->nr_alloc_calls == 0);
+      vg_assert (ddpa->fixedSzb == 0);
+      ddpa->strPA = True;
+   }
+
+   const UInt nr_str = VG_(HT_count_nodes)(ddpa->ht_elements);
+   const ht_node* ht_ins = allocEltDedupPA(ddpa, VG_(strlen)(str)+1, str);
+
+   *newStr = nr_str < VG_(HT_count_nodes)(ddpa->ht_elements);
+   return ht_ins->eltSzBorStrNr;
 }
 
 static __inline__
@@ -303,6 +358,7 @@ UInt VG_(allocFixedEltDedupPA) (DedupPoolAlloc *ddpa,
 {
    if (ddpa->fixedSzb == 0) {
       // First insertion in this ddpa
+      vg_assert (!ddpa->strPA);
       vg_assert (ddpa->nr_alloc_calls == 0);
       vg_assert (eltSzB > 0);
       ddpa->fixedSzb = eltSzB;
