@@ -118,6 +118,23 @@ void VG_(get_UnwindStartRegs) ( /*OUT*/UnwindStartRegs* regs,
       = VG_(threads)[tid].arch.vex.guest_FP;
    regs->misc.S390X.r_lr
       = VG_(threads)[tid].arch.vex.guest_LR;
+   /* ANDREAS 3 Apr 2019 FIXME r_f0..r_f7: is this correct? */
+   regs->misc.S390X.r_f0
+      = VG_(threads)[tid].arch.vex.guest_v0.w64[0];
+   regs->misc.S390X.r_f1
+      = VG_(threads)[tid].arch.vex.guest_v1.w64[0];
+   regs->misc.S390X.r_f2
+      = VG_(threads)[tid].arch.vex.guest_v2.w64[0];
+   regs->misc.S390X.r_f3
+      = VG_(threads)[tid].arch.vex.guest_v3.w64[0];
+   regs->misc.S390X.r_f4
+      = VG_(threads)[tid].arch.vex.guest_v4.w64[0];
+   regs->misc.S390X.r_f5
+      = VG_(threads)[tid].arch.vex.guest_v5.w64[0];
+   regs->misc.S390X.r_f6
+      = VG_(threads)[tid].arch.vex.guest_v6.w64[0];
+   regs->misc.S390X.r_f7
+      = VG_(threads)[tid].arch.vex.guest_v7.w64[0];
 #  elif defined(VGA_mips32)
    regs->r_pc = VG_(threads)[tid].arch.vex.guest_PC;
    regs->r_sp = VG_(threads)[tid].arch.vex.guest_r29;
@@ -757,9 +774,11 @@ static Bool VG_(parse_cpuinfo)(void)
           case VEX_PRID_COMP_CAVIUM:
           case VEX_PRID_COMP_NETLOGIC:
              vai.hwcaps |= (VEX_MIPS_CPU_ISA_M64R2 | VEX_MIPS_CPU_ISA_M64R1);
+             /* fallthrough */
           case VEX_PRID_COMP_INGENIC_E1:
           case VEX_PRID_COMP_MIPS:
              vai.hwcaps |= VEX_MIPS_CPU_ISA_M32R2;
+             /* fallthrough */
           case VEX_PRID_COMP_BROADCOM:
              vai.hwcaps |= VEX_MIPS_CPU_ISA_M32R1;
              break;
@@ -943,14 +962,20 @@ Bool VG_(machine_get_hwcaps)( void )
    }
 
 #elif defined(VGA_amd64)
-   { Bool have_sse3, have_cx8, have_cx16;
+   { Bool have_sse3, have_ssse3, have_cx8, have_cx16;
      Bool have_lzcnt, have_avx, have_bmi, have_avx2;
-     Bool have_rdtscp;
+     Bool have_rdtscp, have_rdrand, have_f16c;
      UInt eax, ebx, ecx, edx, max_basic, max_extended;
      ULong xgetbv_0 = 0;
      HChar vstr[13];
      vstr[0] = 0;
 
+     have_sse3 = have_ssse3 = have_cx8 = have_cx16
+               = have_lzcnt = have_avx = have_bmi = have_avx2
+               = have_rdtscp = have_rdrand = have_f16c = False;
+
+     eax = ebx = ecx = edx = max_basic = max_extended = 0;
+
      if (!VG_(has_cpuid)())
         /* we can't do cpuid at all.  Give up. */
         return False;
@@ -975,15 +1000,17 @@ Bool VG_(machine_get_hwcaps)( void )
      VG_(cpuid)(1, 0, &eax, &ebx, &ecx, &edx);
 
      // we assume that SSE1 and SSE2 are available by default
-     have_sse3 = (ecx & (1<<0)) != 0;  /* True => have sse3 insns */
-     // ssse3   is ecx:9
+     have_sse3  = (ecx & (1<<0)) != 0;  /* True => have sse3 insns */
+     have_ssse3 = (ecx & (1<<9)) != 0;  /* True => have Sup SSE3 insns */
+     // fma     is ecx:12
      // sse41   is ecx:19
      // sse42   is ecx:20
-
      // xsave   is ecx:26
      // osxsave is ecx:27
      // avx     is ecx:28
-     // fma     is ecx:12
+     have_f16c   = (ecx & (1<<29)) != 0; /* True => have F16C insns */
+     have_rdrand = (ecx & (1<<30)) != 0; /* True => have RDRAND insns */
+
      have_avx = False;
      /* have_fma = False; */
      if ( (ecx & ((1<<28)|(1<<27)|(1<<26))) == ((1<<28)|(1<<27)|(1<<26)) ) {
@@ -1051,15 +1078,26 @@ Bool VG_(machine_get_hwcaps)( void )
         have_avx2 = (ebx & (1<<5)) != 0; /* True => have AVX2 */
      }
 
+     /* Sanity check for RDRAND and F16C.  These don't actually *need* AVX2, but
+        it's convenient to restrict them to the AVX2 case since the simulated
+        CPUID we'll offer them on has AVX2 as a base. */
+     if (!have_avx2) {
+        have_f16c   = False;
+        have_rdrand = False;
+     }
+
      va          = VexArchAMD64;
      vai.endness = VexEndnessLE;
      vai.hwcaps  = (have_sse3   ? VEX_HWCAPS_AMD64_SSE3   : 0)
+                 | (have_ssse3  ? VEX_HWCAPS_AMD64_SSSE3  : 0)
                  | (have_cx16   ? VEX_HWCAPS_AMD64_CX16   : 0)
                  | (have_lzcnt  ? VEX_HWCAPS_AMD64_LZCNT  : 0)
                  | (have_avx    ? VEX_HWCAPS_AMD64_AVX    : 0)
                  | (have_bmi    ? VEX_HWCAPS_AMD64_BMI    : 0)
                  | (have_avx2   ? VEX_HWCAPS_AMD64_AVX2   : 0)
-                 | (have_rdtscp ? VEX_HWCAPS_AMD64_RDTSCP : 0);
+                 | (have_rdtscp ? VEX_HWCAPS_AMD64_RDTSCP : 0)
+                 | (have_f16c   ? VEX_HWCAPS_AMD64_F16C   : 0)
+                 | (have_rdrand ? VEX_HWCAPS_AMD64_RDRAND : 0);
 
      VG_(machine_get_cache_info)(&vai);
 
