@@ -178,22 +178,22 @@ static void save_sigregs(ThreadState *tst, _vki_sigregs *sigregs)
    sigregs->regs.acrs[14] = tst->arch.vex.guest_a14;
    sigregs->regs.acrs[15] = tst->arch.vex.guest_a15;
 
-   sigregs->fpregs.fprs[0] = tst->arch.vex.guest_f0;
-   sigregs->fpregs.fprs[1] = tst->arch.vex.guest_f1;
-   sigregs->fpregs.fprs[2] = tst->arch.vex.guest_f2;
-   sigregs->fpregs.fprs[3] = tst->arch.vex.guest_f3;
-   sigregs->fpregs.fprs[4] = tst->arch.vex.guest_f4;
-   sigregs->fpregs.fprs[5] = tst->arch.vex.guest_f5;
-   sigregs->fpregs.fprs[6] = tst->arch.vex.guest_f6;
-   sigregs->fpregs.fprs[7] = tst->arch.vex.guest_f7;
-   sigregs->fpregs.fprs[8] = tst->arch.vex.guest_f8;
-   sigregs->fpregs.fprs[9] = tst->arch.vex.guest_f9;
-   sigregs->fpregs.fprs[10] = tst->arch.vex.guest_f10;
-   sigregs->fpregs.fprs[11] = tst->arch.vex.guest_f11;
-   sigregs->fpregs.fprs[12] = tst->arch.vex.guest_f12;
-   sigregs->fpregs.fprs[13] = tst->arch.vex.guest_f13;
-   sigregs->fpregs.fprs[14] = tst->arch.vex.guest_f14;
-   sigregs->fpregs.fprs[15] = tst->arch.vex.guest_f15;
+   sigregs->fpregs.fprs[0] = *((const Double *) &tst->arch.vex.guest_v0.w64[0]);
+   sigregs->fpregs.fprs[1] = *((const Double *) &tst->arch.vex.guest_v1.w64[0]);
+   sigregs->fpregs.fprs[2] = *((const Double *) &tst->arch.vex.guest_v2.w64[0]);
+   sigregs->fpregs.fprs[3] = *((const Double *) &tst->arch.vex.guest_v3.w64[0]);
+   sigregs->fpregs.fprs[4] = *((const Double *) &tst->arch.vex.guest_v4.w64[0]);
+   sigregs->fpregs.fprs[5] = *((const Double *) &tst->arch.vex.guest_v5.w64[0]);
+   sigregs->fpregs.fprs[6] = *((const Double *) &tst->arch.vex.guest_v6.w64[0]);
+   sigregs->fpregs.fprs[7] = *((const Double *) &tst->arch.vex.guest_v7.w64[0]);
+   sigregs->fpregs.fprs[8] = *((const Double *) &tst->arch.vex.guest_v8.w64[0]);
+   sigregs->fpregs.fprs[9] = *((const Double *) &tst->arch.vex.guest_v9.w64[0]);
+   sigregs->fpregs.fprs[10] = *((const Double *) &tst->arch.vex.guest_v10.w64[0]);
+   sigregs->fpregs.fprs[11] = *((const Double *) &tst->arch.vex.guest_v11.w64[0]);
+   sigregs->fpregs.fprs[12] = *((const Double *) &tst->arch.vex.guest_v12.w64[0]);
+   sigregs->fpregs.fprs[13] = *((const Double *) &tst->arch.vex.guest_v13.w64[0]);
+   sigregs->fpregs.fprs[14] = *((const Double *) &tst->arch.vex.guest_v14.w64[0]);
+   sigregs->fpregs.fprs[15] = *((const Double *) &tst->arch.vex.guest_v15.w64[0]);
    sigregs->fpregs.fpc      = tst->arch.vex.guest_fpc;
 
    sigregs->regs.psw.addr = tst->arch.vex.guest_IA;
@@ -237,22 +237,22 @@ static void restore_sigregs(ThreadState *tst, _vki_sigregs *sigregs)
    tst->arch.vex.guest_a14 = sigregs->regs.acrs[14];
    tst->arch.vex.guest_a15 = sigregs->regs.acrs[15];
 
-   tst->arch.vex.guest_f0  = sigregs->fpregs.fprs[0];
-   tst->arch.vex.guest_f1  = sigregs->fpregs.fprs[1];
-   tst->arch.vex.guest_f2  = sigregs->fpregs.fprs[2];
-   tst->arch.vex.guest_f3  = sigregs->fpregs.fprs[3];
-   tst->arch.vex.guest_f4  = sigregs->fpregs.fprs[4];
-   tst->arch.vex.guest_f5  = sigregs->fpregs.fprs[5];
-   tst->arch.vex.guest_f6  = sigregs->fpregs.fprs[6];
-   tst->arch.vex.guest_f7  = sigregs->fpregs.fprs[7];
-   tst->arch.vex.guest_f8  = sigregs->fpregs.fprs[8];
-   tst->arch.vex.guest_f9  = sigregs->fpregs.fprs[9];
-   tst->arch.vex.guest_f10 = sigregs->fpregs.fprs[10];
-   tst->arch.vex.guest_f11 = sigregs->fpregs.fprs[11];
-   tst->arch.vex.guest_f12 = sigregs->fpregs.fprs[12];
-   tst->arch.vex.guest_f13 = sigregs->fpregs.fprs[13];
-   tst->arch.vex.guest_f14 = sigregs->fpregs.fprs[14];
-   tst->arch.vex.guest_f15 = sigregs->fpregs.fprs[15];
+   *((Double *) &tst->arch.vex.guest_v0.w64[0])  = sigregs->fpregs.fprs[0];
+   *((Double *) &tst->arch.vex.guest_v1.w64[0])  = sigregs->fpregs.fprs[1];
+   *((Double *) &tst->arch.vex.guest_v2.w64[0])  = sigregs->fpregs.fprs[2];
+   *((Double *) &tst->arch.vex.guest_v3.w64[0])  = sigregs->fpregs.fprs[3];
+   *((Double *) &tst->arch.vex.guest_v4.w64[0])  = sigregs->fpregs.fprs[4];
+   *((Double *) &tst->arch.vex.guest_v5.w64[0])  = sigregs->fpregs.fprs[5];
+   *((Double *) &tst->arch.vex.guest_v6.w64[0])  = sigregs->fpregs.fprs[6];
+   *((Double *) &tst->arch.vex.guest_v7.w64[0])  = sigregs->fpregs.fprs[7];
+   *((Double *) &tst->arch.vex.guest_v8.w64[0])  = sigregs->fpregs.fprs[8];
+   *((Double *) &tst->arch.vex.guest_v9.w64[0])  = sigregs->fpregs.fprs[9];
+   *((Double *) &tst->arch.vex.guest_v10.w64[0]) = sigregs->fpregs.fprs[10];
+   *((Double *) &tst->arch.vex.guest_v11.w64[0]) = sigregs->fpregs.fprs[11];
+   *((Double *) &tst->arch.vex.guest_v12.w64[0]) = sigregs->fpregs.fprs[12];
+   *((Double *) &tst->arch.vex.guest_v13.w64[0]) = sigregs->fpregs.fprs[13];
+   *((Double *) &tst->arch.vex.guest_v14.w64[0]) = sigregs->fpregs.fprs[14];
+   *((Double *) &tst->arch.vex.guest_v15.w64[0]) = sigregs->fpregs.fprs[15];
    tst->arch.vex.guest_fpc = sigregs->fpregs.fpc;
 
    tst->arch.vex.guest_IA = sigregs->regs.psw.addr;
