@@ -293,6 +293,14 @@ DECL_TEMPLATE(linux, sys_memfd_create);
 
 DECL_TEMPLATE(linux, sys_syncfs);
 
+DECL_TEMPLATE(linux, sys_membarrier);
+
+// Linux-specific (new in Linux 3.18)
+DECL_TEMPLATE(linux, sys_bpf);
+
+// Linux-specific (new in Linux 4.11)
+DECL_TEMPLATE(linux, sys_statx);
+
 /* ---------------------------------------------------------------------
    Wrappers for sockets and ipc-ery.  These are split into standalone
    procedures because x86-linux hides them inside multiplexors
