@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2012 Julian Seward
+   Copyright (C) 2000-2017 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -40,6 +40,7 @@
 //--------------------------------------------------------------------
 
 #include "pub_core_dispatch_asm.h"
+#include "pub_core_basics.h"        // Addr
 
 /* Run translations, with the given guest state, and starting by
    running the host code at 'host_addr'.  It is almost always the case
@@ -61,7 +62,7 @@
    where we have to return a chain-me request.
 */
 void VG_(disp_run_translations)( HWord* two_words,
-                                 void*  guest_state, 
+                                 volatile void*  guest_state, 
                                  Addr   host_addr );
 
 /* We need to know addresses of the continuation-point (cp_) labels so
