@@ -47,6 +47,12 @@
 // If with_stacktraces, outputs all the recorded stacktraces.
 extern void VG_(print_ExeContext_stats) ( Bool with_stacktraces );
 
+// All ExeContext that are valid in the current epoch and have one or more
+// ips in the given range are archived, i.e. their epoch is frozen to
+// the given last_epoch.
+extern void VG_(archive_ExeContext_in_range) (DiEpoch last_epoch,
+                                              Addr text_avma, SizeT length );
+
 // Extract the StackTrace from an ExeContext.
 // (Minor hack: we use Addr* as the return type instead of StackTrace so
 // that modules #including this file don't also have to #include
