@@ -79,20 +79,6 @@ extern void VG_(unimplemented) ( const HChar* format, ... )
             __attribute__((__noreturn__))
             PRINTF_CHECK(1, 2);
 
-/* Show the state of various threads related information, such
-   as the guest stacktrace for each thread.
-   Mostly for debugging V.
-   The following activates optional output:
-     host_stacktrace : shows the host stacktrace.
-     stack_usage True means:
-                   shows how much of the valgrind stack was used.
-                   shows the client stack range
-     exited_thread_slots : show information for thread slots that were used
-        but the thread has now exited. */
-extern void VG_(show_sched_status) ( Bool host_stacktrace,
-                                     Bool stack_usage,
-                                     Bool exited_threads);
-
 #endif   // __PUB_CORE_LIBCASSERT_H
 
 /*--------------------------------------------------------------------*/
