@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2015 Julian Seward
+   Copyright (C) 2000-2017 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -156,7 +156,7 @@ typedef struct {
    void (*tool_print_stats)(void);
 
    // VG_(needs).info_location
-   void (*tool_info_location)(Addr a);
+   void (*tool_info_location)(DiEpoch ep, Addr a);
 
    // VG_(needs).malloc_replacement
    void* (*tool_malloc)              (ThreadId, SizeT);
@@ -211,6 +211,8 @@ typedef struct {
    void VG_REGPARM(1) (*track_new_mem_stack_160)(Addr);
    void (*track_new_mem_stack)(Addr,SizeT);
 
+   Bool any_new_mem_stack; // True if one or more track_new_mem_stack is set
+
    void VG_REGPARM(1) (*track_die_mem_stack_4)  (Addr);
    void VG_REGPARM(1) (*track_die_mem_stack_8)  (Addr);
    void VG_REGPARM(1) (*track_die_mem_stack_12) (Addr);
@@ -222,6 +224,8 @@ typedef struct {
    void VG_REGPARM(1) (*track_die_mem_stack_160)(Addr);
    void (*track_die_mem_stack)(Addr, SizeT);
 
+   Bool any_die_mem_stack; // True if one or more track_die_mem_stack is set
+
    void (*track_ban_mem_stack)(Addr, SizeT);
 
    void (*track_pre_mem_read)       (CorePart, ThreadId, const HChar*, Addr, SizeT);
@@ -255,7 +259,9 @@ extern VgToolInterface VG_(tdict);
    Miscellaneous functions
    ------------------------------------------------------------------ */
 
-Bool VG_(sanity_check_needs) ( const HChar** failmsg );
+/* Sanity checks and finish the initialisation of the tool needs.
+   Returns False and sets a failmsg if the needs are inconsistent. */
+Bool VG_(finish_needs_init) ( const HChar** failmsg );
 
 #endif   // __PUB_CORE_TOOLIFACE_H
 
