@@ -1,7 +1,7 @@
 /*
   This file is part of drd, a thread error detector.
 
-  Copyright (C) 2006-2015 Bart Van Assche <bvanassche@acm.org>.
+  Copyright (C) 2006-2017 Bart Van Assche <bvanassche@acm.org>.
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
@@ -68,8 +68,7 @@ void DRD_(clientreq_init)(void)
  * DRD's handler for Valgrind client requests. The code below handles both
  * DRD's public and tool-internal client requests.
  */
-#if defined(VGP_mips32_linux) || defined(VGP_mips64_linux) || \
-    defined(VGP_tilegx_linux)
+#if defined(VGP_mips32_linux) || defined(VGP_mips64_linux)
  /* There is a cse related issue in gcc for MIPS. Optimization level
     has to be lowered, so cse related optimizations are not
     included. */
@@ -80,12 +79,12 @@ static Bool handle_client_request(ThreadId vg_tid, UWord* arg, UWord* ret)
    UWord result = 0;
    const DrdThreadId drd_tid = DRD_(thread_get_running_tid)();
 
-   tl_assert(vg_tid == VG_(get_running_tid()));
+   tl_assert(vg_tid == VG_(get_running_tid)());
    tl_assert(DRD_(VgThreadIdToDrdThreadId)(vg_tid) == drd_tid
              || (VG_USERREQ__GDB_MONITOR_COMMAND == arg[0]
                  && vg_tid == VG_INVALID_THREADID));
    /* Check the consistency of vg_tid and drd_tid, unless
-      vgdb has forced the invokation of a gdb monitor cmd
+      vgdb has forced the invocation of a gdb monitor cmd
       when no threads was running (i.e. all threads blocked
       in a syscall. In such a case, vg_tid is invalid,
       its conversion to a drd thread id gives also an invalid
