@@ -570,13 +570,13 @@ void DRD_(thread_finished)(const DrdThreadId tid)
 }
 
 /** Called just after fork() in the child process. */
-void DRD_(drd_thread_atfork_child)(const DrdThreadId tid)
+void DRD_(drd_thread_atfork_child)(const ThreadId tid)
 {
    unsigned i;
 
    for (i = 1; i < DRD_N_THREADS; i++)
    {
-      if (i == tid)
+      if (DRD_(g_threadinfo)[i].vg_threadid == tid)
 	 continue;
       if (DRD_(IsValidDrdThreadId(i)))
 	 DRD_(thread_delete)(i, True);
