@@ -1,6 +1,6 @@
 /* Test whether assigning names to threads works properly. */
 
-
+#define _GNU_SOURCE
 #include <pthread.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -11,6 +11,7 @@
 #define NUM_THREADS 10
 
 
+static pthread_barrier_t s_barrier;
 static pthread_mutex_t s_mutex;
 static pthread_cond_t  s_cond;
 static int s_counter;
@@ -27,6 +28,8 @@ static void* thread_func(void* argp)
 
   memset(&invalid_mutex, 0xff, sizeof(invalid_mutex));
 
+  pthread_barrier_wait(&s_barrier);
+
   pthread_mutex_lock(&s_mutex);
   while (s_counter != thread_num)
     pthread_cond_wait(&s_cond, &s_mutex);
@@ -45,6 +48,7 @@ int main(int arg, char** argv)
   int i;
   pthread_t tid[NUM_THREADS];
 
+  pthread_barrier_init(&s_barrier, NULL, NUM_THREADS);
   pthread_mutex_init(&s_mutex, 0);
   pthread_cond_init(&s_cond, 0);
 
