@@ -234,15 +234,15 @@ UWord do_acasW ( UWord* addr, UWord expected, UWord nyu )
       "lw     $t2, 8(%1)"        "\n\t"
       "lw     $t3, 4(%1)"        "\n\t"
       "ll     $t1, 0($t0)"       "\n\t"
-      "bne    $t1, $t2, exit_0"  "\n\t"
+      "bne    $t1, $t2, 1f"      "\n\t"
       "nop"                      "\n\t"
       "sc     $t3, 0($t0)"       "\n\t"
       "move   %0, $t3"           "\n\t"
-      "b exit"                   "\n\t"
+      "b 2f"                     "\n\t"
       "nop"                      "\n\t"
-      "exit_0:"                  "\n\t"
+      "1:"                       "\n\t"
       "move   %0, $zero"         "\n\t"
-      "exit:"                    "\n\t"
+      "2:"                       "\n\t"
       : /*out*/ "=r"(success)
       : /*in*/ "r"(&block[0])
       : /*trash*/ "t0", "t1", "t2", "t3", "memory"
@@ -267,15 +267,15 @@ UWord do_acasW ( UWord* addr, UWord expected, UWord nyu )
       "ld     $t2, 16(%1)"       "\n\t"
       "ld     $t3, 8(%1)"        "\n\t"
       "ll     $t1, 0($t0)"       "\n\t"
-      "bne    $t1, $t2, exit_0"  "\n\t"
+      "bne    $t1, $t2, 1f"      "\n\t"
       "nop"                      "\n\t"
       "sc     $t3, 0($t0)"       "\n\t"
       "move   %0, $t3"           "\n\t"
-      "b exit"                   "\n\t"
+      "b 2f"                     "\n\t"
       "nop"                      "\n\t"
-      "exit_0:"                  "\n\t"
+      "1:"                       "\n\t"
       "move   %0, $zero"         "\n\t"
-      "exit:"                    "\n\t"
+      "2:"                       "\n\t"
       : /*out*/ "=r"(success)
       : /*in*/ "r"(&block[0])
       : /*trash*/ "t0", "t1", "t2", "t3", "memory"
