@@ -2,12 +2,15 @@
 #include <pthread.h>
 #include <stdlib.h>
 #include <unistd.h>
-
+#include <assert.h>
 /* Naive dining philosophers with inconsistent lock acquisition
    ordering. */
 
 static pthread_t phil[5];
-static pthread_mutex_t chop[5];
+static struct {
+   pthread_mutex_t m;
+   char pad[120 - sizeof(pthread_mutex_t)];
+} chop[5];
 
 void* dine ( void* arg )
 {
@@ -15,11 +18,11 @@ void* dine ( void* arg )
    long left = (long)arg;
    long right = (left + 1) % 5;
    for (i = 0; i < 1000/*arbitrary*/; i++) {
-      pthread_mutex_lock(&chop[left]);
-      pthread_mutex_lock(&chop[right]);
+      pthread_mutex_lock(&chop[left].m);
+      pthread_mutex_lock(&chop[right].m);
       /* eating */
-      pthread_mutex_unlock(&chop[left]);
-      pthread_mutex_unlock(&chop[right]);
+      pthread_mutex_unlock(&chop[left].m);
+      pthread_mutex_unlock(&chop[right].m);
    }
    return NULL;
 }
@@ -27,8 +30,10 @@ void* dine ( void* arg )
 int main ( void )
 {
    long i;
+   assert(sizeof(pthread_mutex_t) <= 120);
+
    for (i = 0; i < 5; i++)
-      pthread_mutex_init( &chop[i], NULL);
+      pthread_mutex_init( &chop[i].m, NULL);
 
    for (i = 0; i < 5; i++)
       pthread_create(&phil[i], NULL, dine, (void*)i );
