@@ -6,7 +6,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2013 Julian Seward
+   Copyright (C) 2000-2017 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -30,7 +30,13 @@
 #ifndef __PUB_TOOL_EXECONTEXT_H
 #define __PUB_TOOL_EXECONTEXT_H
 
-#include "pub_tool_basics.h"   // ThreadID
+#include "pub_tool_basics.h"     // ThreadID
+#include "pub_tool_debuginfo.h"  // DiEpoch
+
+
+/*====================================================================*/
+/*=== ExeContext                                                   ===*/
+/*====================================================================*/
 
 // It's an abstract type.
 typedef
@@ -74,14 +80,16 @@ ExeContext* VG_(record_depth_1_ExeContext)(ThreadId tid, Word first_ip_delta);
 // Apply a function to every element in the ExeContext.  The parameter 'n'
 // gives the index of the passed ip.  Doesn't go below main() unless
 // --show-below-main=yes is set.
-extern void VG_(apply_ExeContext)( void(*action)(UInt n, Addr ip),
-                                   ExeContext* ec, UInt n_ips );
+extern void VG_(apply_ExeContext)(
+   void(*action)(UInt n, DiEpoch ep, Addr ip, void* opaque),
+   void* opaque, ExeContext* ec);
 
 // Compare two ExeContexts.  Number of callers considered depends on `res':
 //   Vg_LowRes:  2
 //   Vg_MedRes:  4
 //   Vg_HighRes: all
-extern Bool VG_(eq_ExeContext) ( VgRes res, ExeContext* e1, ExeContext* e2 );
+extern Bool VG_(eq_ExeContext) ( VgRes res, const ExeContext* e1,
+                                 const ExeContext* e2 );
 
 // Print an ExeContext.
 extern void VG_(pp_ExeContext) ( ExeContext* ec );
@@ -90,10 +98,13 @@ extern void VG_(pp_ExeContext) ( ExeContext* ec );
 // (the "ExeContext Unique").  Guaranteed to be nonzero and to be a
 // multiple of four (iow, the lowest two bits are guaranteed to
 // be zero, so that callers can store other information there.
-extern UInt VG_(get_ECU_from_ExeContext)( ExeContext* e );
+extern UInt VG_(get_ECU_from_ExeContext)( const ExeContext* e );
+
+// Returns the epoch in which the ips of e can be symbolised.
+extern DiEpoch VG_(get_ExeContext_epoch)( const ExeContext* e );
 
 // How many entries (frames) in this ExeContext?
-extern Int VG_(get_ExeContext_n_ips)( ExeContext* e );
+extern Int VG_(get_ExeContext_n_ips)( const ExeContext* e );
 
 // Find the ExeContext that has the given ECU, if any.
 // NOTE: very slow.  Do not call often.
@@ -110,7 +121,7 @@ static inline Bool VG_(is_plausible_ECU)( UInt ecu ) {
 }
 
 // Make an ExeContext containing exactly the specified stack frames.
-ExeContext* VG_(make_ExeContext_from_StackTrace)( Addr* ips, UInt n_ips );
+ExeContext* VG_(make_ExeContext_from_StackTrace)( const Addr* ips, UInt n_ips );
 
 // Returns the "null" exe context. The null execontext is an artificial
 // exe context, with a stack trace made of one Addr (the NULL address).
