@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2015 Julian Seward
+   Copyright (C) 2000-2017 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -98,12 +98,17 @@ extern Int VG_(getegid) ( void );
    Timing
    ------------------------------------------------------------------ */
 
-// Returns the number of milliseconds passed since the progam started
+// Returns the number of milliseconds passed since the program started
 // (roughly;  it gets initialised partway through Valgrind's initialisation
-// steps).
+// steps).  This is wallclock time.
 extern UInt VG_(read_millisecond_timer) ( void );
+
 extern Int  VG_(gettimeofday)(struct vki_timeval *tv, struct vki_timezone *tz);
 
+// Returns the number of milliseconds of user cpu time we have used,
+// as reported by 'getrusage'.
+extern UInt VG_(get_user_milliseconds)(void);
+
 /* ---------------------------------------------------------------------
    atfork
    ------------------------------------------------------------------ */
