@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2015 Julian Seward
+   Copyright (C) 2000-2017 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 #ifndef __PUB_TOOL_STACKTRACE_H
 #define __PUB_TOOL_STACKTRACE_H
 
-#include "pub_tool_basics.h"   // Addr
+#include "pub_tool_basics.h"   // Addr, DiEpoch
 
 // The basic stack trace type:  just an array of code addresses.
 typedef Addr* StackTrace;
@@ -64,19 +64,30 @@ extern UInt VG_(get_StackTrace) ( ThreadId tid,
                                   /*OUT*/StackTrace fps,
                                   Word first_ip_delta );
 
-// Apply a function to every element in the StackTrace.  The parameter
-// 'n' gives the index of the passed ip.  'opaque' is an arbitrary
-// pointer provided to each invokation of 'action' (a poor man's
-// closure).  Doesn't go below main() unless --show-below-main=yes is
-// set.
+// Same as VG_(get_StackTrace), but applies a delta to the first SP used for
+//  unwinding the first frame.
+extern UInt VG_(get_StackTrace_with_deltas)(
+                ThreadId tid,
+                /*OUT*/StackTrace ips, UInt n_ips,
+                /*OUT*/StackTrace sps,
+                /*OUT*/StackTrace fps,
+                Word first_ip_delta,
+                Word first_sp_delta
+             );
+
+// Apply a function to every element in the StackTrace.  The parameter 'n'
+// gives the index of the passed ip.  'opaque' is an arbitrary pointer
+// provided to each invocation of 'action' (a poor man's closure).  'ep' is
+// the debuginfo epoch assumed to apply to all code addresses in the stack
+// trace.  Doesn't go below main() unless --show-below-main=yes is set.
 extern void VG_(apply_StackTrace)(
-               void(*action)(UInt n, Addr ip, void* opaque),
+               void(*action)(UInt n, DiEpoch ep, Addr ip, void* opaque),
                void* opaque,
-               StackTrace ips, UInt n_ips
+               DiEpoch ep, StackTrace ips, UInt n_ips
             );
 
 // Print a StackTrace.
-extern void VG_(pp_StackTrace) ( StackTrace ips, UInt n_ips );
+extern void VG_(pp_StackTrace) ( DiEpoch ep, StackTrace ips, UInt n_ips );
 
 // Gets and immediately prints a StackTrace.  Just a bit simpler than
 // calling VG_(get_StackTrace)() then VG_(pp_StackTrace)().
