@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2007-2015 OpenWorks LLP
+   Copyright (C) 2007-2017 OpenWorks LLP
       info@open-works.co.uk
 
    This program is free software; you can redistribute it and/or
@@ -54,9 +54,9 @@ typedef Int (*XACmpFn_t)(const void *, const void *);
    for elements of the specified size.  alloc_fn must not return NULL (that
    is, if it returns it must have succeeded.)
    This function never returns NULL. */
-extern XArray* VG_(newXA) ( void*(*alloc_fn)(const HChar*,SizeT), 
+extern XArray* VG_(newXA) ( Alloc_Fn_t alloc_fn,
                             const HChar* cc,
-                            void(*free_fn)(void*),
+                            Free_Fn_t free_fn,
                             Word elemSzB );
 
 /* Free all memory associated with an XArray. */
@@ -142,6 +142,14 @@ extern void VG_(removeIndexXA)( XArray*, Word );
    specified element, in the array. */
 extern void VG_(insertIndexXA)( XArray*, Word, const void* elem );
 
+/* Replace the element of an XArray at the given index with a copy
+   of the new element.  This is an O(1) operation.
+   Compared to the caller doing:
+          *(T*)VG_(indexXA)(arr, index) = new_value;
+   this function will also mark the array as unsorted.  */
+extern void VG_(replaceIndexXA)( XArray*, Word, const void* elem );
+
+
 /* Make a new, completely independent copy of the given XArray, using
    the existing allocation function to allocate the new space.
    Space for the clone (and all additions to it) is billed to 'cc' unless
@@ -166,6 +174,8 @@ extern void VG_(getContentsXA_UNSAFE)( XArray* sr,
 extern void VG_(xaprintf)( XArray* dst, const HChar* format, ... )
                          PRINTF_CHECK(2, 3);
 
+/* Convenience function: linear search in an XArray of HChar*. */
+extern Bool VG_(strIsMemberXA)(const XArray* xa, const HChar* str );
 #endif   // __PUB_TOOL_XARRAY_H
 
 /*--------------------------------------------------------------------*/
