@@ -1,3 +1,4 @@
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 int main()
@@ -5,7 +6,8 @@ int main()
    __attribute__((unused)) char *p = malloc (1);
    char *b1 = malloc (128);
    char *b2 = malloc (128);
-   fprintf (stderr, "b1 %p b2 %p\n", b1, b2);
+   fprintf (stderr, "b1 %#" PRIxPTR " b2 %#" PRIxPTR "\n",
+                    (uintptr_t)b1, (uintptr_t)b2);
 
    // Try to land in b2 from b1, causing no error
    // with the default redzone-size, but having
