@@ -108,6 +108,24 @@ asm(
 ".previous\n"
 );
 
+#elif defined(VGP_arm64_linux)
+extern UWord do_syscall_WRK (
+          UWord a1, UWord a2, UWord a3,
+          UWord a4, UWord a5, UWord a6,
+          UWord syscall_no
+       );
+asm(
+".text\n"
+".globl do_syscall_WRK\n"
+"do_syscall_WRK:\n"
+"        mov x8, x6\n"
+"        mov x6, 0\n"
+"        mov x7, 0\n"
+"        svc 0\n"
+"        ret\n"
+".previous\n"
+);
+
 #elif defined(VGP_s390x_linux)
 UWord do_syscall_WRK (
    UWord syscall_no,
@@ -163,30 +181,6 @@ extern UWord do_syscall_WRK (
                  : "v0", "v1", "a0", "a1", "a2", "a3", "$8", "$9");
    return out;
 }
-#elif defined(VGP_tilegx_linux)
-extern UWord do_syscall_WRK (
-          UWord syscall_no,
-          UWord a1, UWord a2, UWord a3,
-          UWord a4, UWord a5, UWord a6
-       )
-{
-   UWord out;
-   __asm__ __volatile__ (
-                 "move r10, %1\n\t"
-                 "move r0,  %2\n\t"
-                 "move r1,  %3\n\t"
-                 "move r2,  %4\n\t"
-                 "move r3,  %5\n\t"
-                 "move r4,  %6\n\t"
-                 "move r5,  %7\n\t"
-                 "swint1      \n\t"
-                 "move %0,  r0\n\t"
-                 : /*out*/ "=r" (out)
-                 : "r"(syscall_no), "r"(a1), "r"(a2), "r"(a3),
-                   "r"(a4), "r"(a5), "r"(a6)
-                 : "r0", "r1", "r2", "r3", "r4", "r5", "r6", "r10");
-   return out;
-}
 
 #elif defined(VGP_x86_solaris)
 extern ULong
@@ -262,6 +256,10 @@ static void non_simd_mprotect (long tid, void* addr, long len)
                                     &err);
    if (err)
       mprotect_result = -1;
+#elif defined(VGP_arm64_linux)
+   mprotect_result = do_syscall_WRK((UWord) addr, len, PROT_NONE,
+                                    0, 0, 0,
+                                    __NR_mprotect);
 #else
    mprotect_result = do_syscall_WRK(__NR_mprotect,
                                     (UWord) addr, len, PROT_NONE,
@@ -287,7 +285,7 @@ void f(void)
    long pagesize;
 #define RNDPAGEDOWN(a) ((long)a & ~(pagesize-1))
    int i;
-   const int nr_ptr = (10000 * 4)/sizeof(char*);
+   const int nr_ptr = (10000 * 20)/sizeof(char*);
 
    b10 = calloc (nr_ptr * sizeof(char*), 1);
    for (i = 0; i < nr_ptr; i++)
