@@ -82,10 +82,11 @@
                             "move $15, $0 \n\t"   /* t7 = 0 */               \
                             "move $24, $0 \n\t"   /* t8 = 0 */               \
                             "move $25, $0 \n\t"   /* t9 = 0 */               \
+                            "move $31, $0 \n\t"   /* ra = 0 */               \
                             ".set pop     \n\t"                              \
                             : : : "$1", "$2", "$3", "$4", "$5", "$6", "$7",  \
                                   "$8", "$9", "$10", "$11", "$12", "$13",    \
-                                  "$14", "$15", "$24", "$25");               \
+                                  "$14", "$15", "$24", "$25", "$31");        \
    } while (0)
 #elif (__mips == 64)
 #define CLEAR_CALLER_SAVED_REGS                                              \
@@ -109,10 +110,11 @@
                             "move $15, $0 \n\t"  /* t3 = 0 */                \
                             "move $24, $0 \n\t"  /* t8 = 0 */                \
                             "move $25, $0 \n\t"  /* t9 = 0 */                \
+                            "move $31, $0 \n\t"  /* ra = 0 */                \
                             ".set pop     \n\t"                              \
                             : : : "$1", "$2", "$3", "$4", "$5", "$6", "$7",  \
                                   "$8", "$9", "$10", "$11", "$12", "$13",    \
-                                  "$14", "$15", "$24", "$25");               \
+                                  "$14", "$15", "$24", "$25", "$31");        \
    } while (0)
 #else
 #define CLEAR_CALLER_SAVED_REGS  /*nothing*/
