@@ -111,6 +111,12 @@ static irop_t irops[] = {
   { DEFOP(Iop_Clz32,      UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 1, .arm = 1, .ppc64 = 1, .ppc32 = 1, .mips32 =1, .mips64 = 1 },
   { DEFOP(Iop_Ctz64,      UNDEF_ALL),  .s390x = 0, .amd64 = 1, .x86 = 0, .arm = 0, .ppc64 = 0, .ppc32 = 0, .mips32 =0, .mips64 = 0 },
   { DEFOP(Iop_Ctz32,      UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 1, .arm = 0, .ppc64 = 0, .ppc32 = 0, .mips32 =0, .mips64 = 0 },
+  { DEFOP(Iop_ClzNat64,   UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 0, .arm = 0, .ppc64 = 1, .ppc32 = 0, .mips32 =0, .mips64 = 0 }, // ppc32 asserts
+  { DEFOP(Iop_ClzNat32,   UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 0, .arm = 0, .ppc64 = 1, .ppc32 = 1, .mips32 =0, .mips64 = 0 },
+  { DEFOP(Iop_CtzNat64,   UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 0, .arm = 0, .ppc64 = 1, .ppc32 = 0, .mips32 =0, .mips64 = 0 },
+  { DEFOP(Iop_CtzNat32,   UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 0, .arm = 0, .ppc64 = 0, .ppc32 = 1, .mips32 =0, .mips64 = 0 },
+  { DEFOP(Iop_PopCount64, UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 0, .arm = 0, .ppc64 = 1, .ppc32 = 0, .mips32 =0, .mips64 = 0 },
+  { DEFOP(Iop_PopCount32, UNDEF_ALL),  .s390x = 0, .amd64 = 0, .x86 = 0, .arm = 0, .ppc64 = 1, .ppc32 = 1, .mips32 =0, .mips64 = 0 },
   { DEFOP(Iop_CmpLT32S,   UNDEF_ALL),  .s390x = 1, .amd64 = 1, .x86 = 1, .arm = 1, .ppc64 = 1, .ppc32 = 1, .mips32 =1, .mips64 = 1 },
   { DEFOP(Iop_CmpLT64S,   UNDEF_ALL),  .s390x = 1, .amd64 = 1, .x86 = 0, .arm = 0, .ppc64 = 0, .ppc32 = 0, .mips32 =0, .mips64 = 1 }, // ppc, mips assert
   { DEFOP(Iop_CmpLE32S,   UNDEF_ALL),  .s390x = 1, .amd64 = 1, .x86 = 1, .arm = 1, .ppc64 = 1, .ppc32 = 1, .mips32 =1, .mips64 = 1 },
@@ -336,11 +342,12 @@ static irop_t irops[] = {
   { DEFOP(Iop_Sad8Ux4, UNDEF_UNKNOWN), },
   { DEFOP(Iop_CmpNEZ16x2, UNDEF_UNKNOWN), },
   { DEFOP(Iop_CmpNEZ8x4, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_Reverse8sIn32_x1, UNDEF_UNKNOWN) },
   /* ------------------ 64-bit SIMD FP ------------------------ */
-  { DEFOP(Iop_I32UtoFx2, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_I32StoFx2, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_FtoI32Ux2_RZ, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_FtoI32Sx2_RZ, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_I32UtoF32x2_DEP, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_I32StoF32x2_DEP, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toI32Ux2_RZ, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toI32Sx2_RZ, UNDEF_UNKNOWN), },
   { DEFOP(Iop_F32ToFixed32Ux2_RZ, UNDEF_UNKNOWN), },
   { DEFOP(Iop_F32ToFixed32Sx2_RZ, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Fixed32UToF32x2_RN, UNDEF_UNKNOWN), },
@@ -527,6 +534,7 @@ static irop_t irops[] = {
   { DEFOP(Iop_Reverse16sIn64_x1, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Reverse32sIn64_x1, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Perm8x8, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_PermOrZero8x8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_GetMSBs8x8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RecipEst32Ux2, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RSqrtEst32Ux2, UNDEF_UNKNOWN), },
@@ -630,12 +638,14 @@ static irop_t irops[] = {
   { DEFOP(Iop_RecipStep32Fx4, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RSqrtEst32Fx4, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RSqrtStep32Fx4, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_I32UtoFx4, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_I32StoFx4, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_FtoI32Ux4_RZ, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_FtoI32Sx4_RZ, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_QFtoI32Ux4_RZ, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_QFtoI32Sx4_RZ, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_I32UtoF32x4_DEP, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_I32StoF32x4_DEP, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_I32StoF32x4, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toI32Sx4, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toI32Ux4_RZ, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toI32Sx4_RZ, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_QF32toI32Ux4_RZ, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_QF32toI32Sx4_RZ, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RoundF32x4_RM, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RoundF32x4_RP, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RoundF32x4_RN, UNDEF_UNKNOWN), },
@@ -644,9 +654,10 @@ static irop_t irops[] = {
   { DEFOP(Iop_F32ToFixed32Sx4_RZ, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Fixed32UToF32x4_RN, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Fixed32SToF32x4_RN, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toF16x4_DEP, UNDEF_UNKNOWN), },
   { DEFOP(Iop_F32toF16x4, UNDEF_UNKNOWN), },
   { DEFOP(Iop_F16toF32x4, UNDEF_UNKNOWN), },
-  { DEFOP(Iop_F64toF16x2, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F64toF16x2_DEP, UNDEF_UNKNOWN), },
   { DEFOP(Iop_F16toF64x2, UNDEF_UNKNOWN), },
   { DEFOP(Iop_F32x4_2toQ16x8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_F64x2_2toQ32x4, UNDEF_UNKNOWN), },
@@ -795,6 +806,7 @@ static irop_t irops[] = {
   { DEFOP(Iop_PwAddL8Sx16, UNDEF_UNKNOWN), },
   { DEFOP(Iop_PwAddL16Sx8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_PwAddL32Sx4, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_PwExtUSMulQAdd8x16, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Abs8x16, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Abs16x8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Abs32x4, UNDEF_UNKNOWN), },
@@ -1026,6 +1038,7 @@ static irop_t irops[] = {
   { DEFOP(Iop_Reverse32sIn64_x2, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Reverse1sIn8_x16, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Perm8x16, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_PermOrZero8x16, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Perm32x4, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Perm8x16x2, UNDEF_UNKNOWN), },
   { DEFOP(Iop_GetMSBs8x16, UNDEF_UNKNOWN), },
@@ -1114,6 +1127,10 @@ static irop_t irops[] = {
   { DEFOP(Iop_Sub32Fx8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Mul32Fx8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Div32Fx8, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_I32StoF32x8, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toI32Sx8, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F32toF16x8, UNDEF_UNKNOWN), },
+  { DEFOP(Iop_F16toF32x8, UNDEF_UNKNOWN) },
   { DEFOP(Iop_Sqrt32Fx8, UNDEF_UNKNOWN), },
   { DEFOP(Iop_Sqrt64Fx4, UNDEF_UNKNOWN), },
   { DEFOP(Iop_RSqrtEst32Fx8, UNDEF_UNKNOWN), },
@@ -1317,8 +1334,8 @@ get_irop(IROp op)
          case Iop_TruncF128toI64U:
          case Iop_TruncF128toI64S:
          case Iop_F16toF32x4:
-         case Iop_F32toF16x4:
-         case Iop_F64toF16x2:
+         case Iop_F32toF16x4_DEP:
+         case Iop_F64toF16x2_DEP:
          case Iop_F16toF64x2:
          case Iop_MulI128by10:
          case Iop_MulI128by10Carry:
