@@ -45,6 +45,8 @@
 #include <inttypes.h>
 #include "vtest.h"
 
+#include "memcheck.h" // VALGRIND_MAKE_MEM_DEFINED
+
 
 /* Something bad happened. Cannot continue. */
 void __attribute__((noreturn))
@@ -120,12 +122,14 @@ print_opnd(FILE *fp, const opnd_t *opnd)
 {
    fprintf(fp, "vbits = ");
    print_vbits(fp, opnd->vbits);
-   /* Write the value only if it is defined. Otherwise, there will be error
-      messages about it being undefined */
-   if (equal_vbits(opnd->vbits, defined_vbits(opnd->vbits.num_bits))) {
-      fprintf(fp, "   value = ");
-      print_value(fp, opnd->value, opnd->vbits.num_bits);
-   }
+   /* The value itself might be partially or fully undefined, so take a
+      copy, paint the copy as defined, and print the copied value.  This is
+      so as to avoid error messages from Memcheck, which are correct, but
+      confusing. */
+   volatile value_t value_copy = opnd->value;
+   VALGRIND_MAKE_MEM_DEFINED(&value_copy, sizeof(value_copy));
+   fprintf(fp, "   value = ");
+   print_value(fp, value_copy, opnd->vbits.num_bits);
 }
 
 
