@@ -92,5 +92,10 @@ vbits_t shr_vbits(vbits_t, unsigned amount);
 vbits_t sar_vbits(vbits_t, unsigned amount);
 int     completely_defined_vbits(vbits_t);
 vbits_t cmpord_vbits(unsigned v1_num_bits, unsigned v2_num_bits);
+vbits_t cmp_eq_ne_vbits(vbits_t vbits1, vbits_t vbits2,
+                        value_t val1, value_t val2);
+vbits_t int_add_or_sub_vbits(int isAdd,
+                             vbits_t vbits1, vbits_t vbits2,
+                             value_t val1, value_t val2);
 
 #endif // VBITS_H
