@@ -2,6 +2,16 @@
 
 #include <stdio.h>
 
+/*
+ * Bits 18 (NAN2008) and 19 (ABS2008) are preset by hardware and may differ
+ * between platforms. Hence a macro to clear them before printing FCSR
+ * values.
+ */
+#define FCSR_NAN2008 1 << 18
+#define FCSR_ABS2008 1 << 19
+#define FLAGS_RM_MASK 0xFFFFFFFF & ~(FCSR_ABS2008 | FCSR_NAN2008)
+#define CLEAR_PRESETBITS_FCSR(fcsr) (fcsr & FLAGS_RM_MASK)
+
 typedef enum {
    CEILWS=0, CEILWD,
    FLOORWS, FLOORWD,
@@ -174,42 +184,42 @@ int directedRoundingMode(flt_dir_op_t op) {
          case CEILWS:
               UNOPfw("ceil.w.s");
               printf("%s %d %f\n", flt_dir_op_names[op], fd_w, fs_f[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
          case CEILWD:
               UNOPdw("ceil.w.d");
               printf("%s %d %lf\n", flt_dir_op_names[op], fd_w, fs_d[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
          case FLOORWS:
               UNOPfw("floor.w.s");
               printf("%s %d %f\n", flt_dir_op_names[op], fd_w, fs_f[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
          case FLOORWD:
               UNOPdw("floor.w.d");
               printf("%s %d %lf\n", flt_dir_op_names[op], fd_w, fs_d[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
          case ROUNDWS:
               UNOPfw("round.w.s");
               printf("%s %d %f\n", flt_dir_op_names[op], fd_w, fs_f[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
          case ROUNDWD:
               UNOPdw("round.w.d");
               printf("%s %d %lf\n", flt_dir_op_names[op], fd_w, fs_d[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
          case TRUNCWS:
               UNOPfw("trunc.w.s");
               printf("%s %d %f\n", flt_dir_op_names[op], fd_w, fs_f[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
          case TRUNCWD:
               UNOPdw("trunc.w.d");
               printf("%s %d %lf\n", flt_dir_op_names[op], fd_w, fs_d[i]);
-              printf("fcsr: 0x%x\n", fcsr);
+              printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
               break;
         default:
             printf("error\n");
@@ -237,32 +247,32 @@ int FCSRRoundingMode(flt_round_op_t op1)
             case CVTDS:
                  UNOPfd("cvt.d.s");
                  printf("%s %lf %lf\n", flt_round_op_names[op1], fd_d, fs_f[i]);
-                 printf("fcsr: 0x%x\n", fcsr);
+                 printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
                  break;
             case CVTDW:
                  UNOPwd("cvt.d.w");
                  printf("%s %lf %d\n", flt_round_op_names[op1], fd_d, fs_w[i]);
-                 printf("fcsr: 0x%x\n", fcsr);
+                 printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
                  break;
             case CVTSD:
                  UNOPdf("cvt.s.d");
                  printf("%s %f %lf\n", flt_round_op_names[op1], fd_f, fs_d[i]);
-                 printf("fcsr: 0x%x\n", fcsr);
+                 printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
                  break;
             case CVTSW:
                  UNOPwf("cvt.s.w");
                  printf("%s %f %d\n", flt_round_op_names[op1], fd_f, fs_w[i]);
-                 printf("fcsr: 0x%x\n", fcsr);
+                 printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
                  break;
             case CVTWS:
                  UNOPfw("cvt.w.s");
                  printf("%s %d %f\n", flt_round_op_names[op1], fd_w, fs_f[i]);
-                 printf("fcsr: 0x%x\n", fcsr);
+                 printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
                  break;
             case CVTWD:
                  UNOPdw("cvt.w.d");
                  printf("%s %d %lf\n", flt_round_op_names[op1], fd_w, fs_d[i]);
-                 printf("fcsr: 0x%x\n", fcsr);
+                 printf("fcsr: 0x%x\n", CLEAR_PRESETBITS_FCSR(fcsr));
                  break;
             default:
                  printf("error\n");
