@@ -49,19 +49,32 @@ typedef unsigned int fpscr_t;
 typedef union {
 	float flt;
 	struct {
+#if defined(VGP_ppc64le_linux)
+      unsigned int frac:23;
+      unsigned int exp:8;
+      unsigned int sign:1;
+#else
 		unsigned int sign:1;
 		unsigned int exp:8;
 		unsigned int frac:23;
+#endif
 	} layout;
 } flt_overlay;
 
 typedef union {
 	double dbl;
 	struct {
+#if defined(VGP_ppc64le_linux)
+      unsigned int frac_lo:32;
+      unsigned int frac_hi:20;
+      unsigned int exp:11;
+      unsigned int sign:1;
+#else
 		unsigned int sign:1;
 		unsigned int exp:11;
 		unsigned int frac_hi:20;
 		unsigned int frac_lo:32;
+#endif
 	} layout;
 	struct {
 		unsigned int hi;
@@ -142,7 +155,6 @@ int check_dbl_to_flt_round(round_mode_t mode, double dbl, float *expected)
 	int status = 0;
 	flt_overlay R, E;
 	char *result;
-	char *eq_ne;
 
 	set_rounding_mode(mode);
 
@@ -153,11 +165,9 @@ int check_dbl_to_flt_round(round_mode_t mode, double dbl, float *expected)
 		(R.layout.exp != E.layout.exp) ||
 		(R.layout.frac != E.layout.frac)) {
 		result = "FAILED";
-		eq_ne = "!=";
 		status = 1;
 	} else {
 		result = "PASSED";
-		eq_ne = "==";
 		status = 0;
 	}
 	printf("%s:%s:(double)(%-20a) = %20a",
@@ -230,7 +240,7 @@ init()
 	F.layout.frac = 1;
 	denorm_small = F.flt;	/* == 2^(-149) */
 	if (debug) {
-		print_double("float small", F.flt);
+		print_single("float small", &F.flt);
 	}
 
 	D.layout.sign = 0;
@@ -797,7 +807,21 @@ int check_single_guarded_arithmetic_op(flt_op_t op)
 			status = 0;
 		}
 
-		printf("%s:%s:%s(%-13f",
+		/* There seems to be some noise in the lower bits. The value
+		* on the least significant digit seems to vary when printing
+		* based on the rounding mode of the compiler.  Just trying
+		* to get rid of the noise in the least significant bits when
+		* printing the operand.
+		*/
+
+		fA = ((long int)(fA*10000))/10000.0;
+		/* Change -0.0 to a positive 0.0.  Some compilers print -0.0
+		 * others do not.  Make it consistent.
+		 */
+		if (fA == -0.0)
+		  fA = 0.0;
+
+		printf("%s:%s:%s(%-13.6f",
 			round_mode_name[mode], result, flt_op_names[op], fA);
 		if (arg_count > 1) printf(", %-13a", fB);
 		if (arg_count > 2) printf(", %-13a", fC);
